@@ -77,7 +77,7 @@ sudo_securid_init(struct passwd *pw, sudo_auth *auth)
     if (AceInitialize() != SD_FALSE)
 	debug_return_int(AUTH_SUCCESS);
 
-    warningx(U_("failed to initialise the ACE API library"));
+    sudo_warnx(U_("failed to initialise the ACE API library"));
     debug_return_int(AUTH_FATAL);
 }
 
@@ -103,7 +103,7 @@ sudo_securid_setup(struct passwd *pw, char **promptp, sudo_auth *auth)
 
     /* Re-initialize SecurID every time. */
     if (SD_Init(sd) != ACM_OK) {
-	warningx(U_("unable to contact the SecurID server"));
+	sudo_warnx(U_("unable to contact the SecurID server"));
 	debug_return_int(AUTH_FATAL);
     }
 
@@ -112,23 +112,23 @@ sudo_securid_setup(struct passwd *pw, char **promptp, sudo_auth *auth)
 
     switch (retval) {
 	case ACM_OK:
-		warningx(U_("User ID locked for SecurID Authentication"));
+		sudo_warnx(U_("User ID locked for SecurID Authentication"));
 		debug_return_int(AUTH_SUCCESS);
 
         case ACE_UNDEFINED_USERNAME:
-		warningx(U_("invalid username length for SecurID"));
+		sudo_warnx(U_("invalid username length for SecurID"));
 		debug_return_int(AUTH_FATAL);
 
 	case ACE_ERR_INVALID_HANDLE:
-		warningx(U_("invalid Authentication Handle for SecurID"));
+		sudo_warnx(U_("invalid Authentication Handle for SecurID"));
 		debug_return_int(AUTH_FATAL);
 
 	case ACM_ACCESS_DENIED:
-		warningx(U_("SecurID communication failed"));
+		sudo_warnx(U_("SecurID communication failed"));
 		debug_return_int(AUTH_FATAL);
 
 	default:
-		warningx(U_("unknown SecurID error"));
+		sudo_warnx(U_("unknown SecurID error"));
 		debug_return_int(AUTH_FATAL);
 	}
 }
@@ -162,17 +162,17 @@ sudo_securid_verify(struct passwd *pw, char *pass, sudo_auth *auth)
 		break;
 
 	case ACE_UNDEFINED_PASSCODE:
-		warningx(U_("invalid passcode length for SecurID"));
+		sudo_warnx(U_("invalid passcode length for SecurID"));
 		rval = AUTH_FATAL;
 		break;
 
 	case ACE_UNDEFINED_USERNAME:
-		warningx(U_("invalid username length for SecurID"));
+		sudo_warnx(U_("invalid username length for SecurID"));
 		rval = AUTH_FATAL;
 		break;
 
 	case ACE_ERR_INVALID_HANDLE:
-		warningx(U_("invalid Authentication Handle for SecurID"));
+		sudo_warnx(U_("invalid Authentication Handle for SecurID"));
 		rval = AUTH_FATAL;
 		break;
 
@@ -211,7 +211,7 @@ then enter the new token code.\n", \
 		break;
 
 	default:
-		warningx(U_("unknown SecurID error"));
+		sudo_warnx(U_("unknown SecurID error"));
 		rval = AUTH_FATAL;
 		break;
     }
