@@ -46,12 +46,16 @@
 #ifdef HAVE_STRINGS_H
 # include <strings.h>
 #endif /* HAVE_STRINGS_H */
+#ifdef HAVE_INTTYPES_H
+# include <inttypes.h>
+#endif
 #ifdef TIME_WITH_SYS_TIME
 # include <time.h>
 #endif
+#include <limits.h>
 #include <ctype.h>
 
-#include "missing.h"
+#include "sudo_compat.h"
 
 
 #define EPOCH		1970
@@ -114,7 +118,7 @@ static int	yyerror(const char *s);
 static int	yylex(void);
        int	yyparse(void);
 
-#line 107 "getdate.y"
+#line 111 "getdate.y"
 #ifndef YYSTYPE_DEFINED
 #define YYSTYPE_DEFINED
 typedef union {
@@ -122,7 +126,7 @@ typedef union {
     enum _MERIDIAN	Meridian;
 } YYSTYPE;
 #endif /* YYSTYPE_DEFINED */
-#line 125 "getdate.c"
+#line 129 "getdate.c"
 #define tAGO 257
 #define tDAY 258
 #define tDAYZONE 259
@@ -390,7 +394,7 @@ short *yyss;
 short *yysslim;
 YYSTYPE *yyvs;
 unsigned int yystacksize;
-#line 326 "getdate.y"
+#line 330 "getdate.y"
 
 /* Month and day table. */
 static TABLE const MonthDayTable[] = {
@@ -1012,7 +1016,7 @@ main(int argc, char *argv[])
     /* NOTREACHED */
 }
 #endif	/* TEST */
-#line 963 "getdate.c"
+#line 967 "getdate.c"
 /* allocate initial stack or double stack size, up to YYMAXDEPTH */
 #if defined(__cplusplus) || defined(__STDC__)
 static int yygrowstack(void)
@@ -1146,12 +1150,12 @@ yyloop:
         goto yyreduce;
     }
     if (yyerrflag) goto yyinrecovery;
-#if defined(lint) || defined(__GNUC__)
+#if defined(__GNUC__)
     goto yynewerror;
 #endif
 yynewerror:
     yyerror("syntax error");
-#if defined(lint) || defined(__GNUC__)
+#if defined(__GNUC__)
     goto yyerrlab;
 #endif
 yyerrlab:
@@ -1221,37 +1225,37 @@ yyreduce:
     switch (yyn)
     {
 case 3:
-#line 125 "getdate.y"
+#line 129 "getdate.y"
 {
 	    yyHaveTime++;
 	}
 break;
 case 4:
-#line 128 "getdate.y"
+#line 132 "getdate.y"
 {
 	    yyHaveZone++;
 	}
 break;
 case 5:
-#line 131 "getdate.y"
+#line 135 "getdate.y"
 {
 	    yyHaveDate++;
 	}
 break;
 case 6:
-#line 134 "getdate.y"
+#line 138 "getdate.y"
 {
 	    yyHaveDay++;
 	}
 break;
 case 7:
-#line 137 "getdate.y"
+#line 141 "getdate.y"
 {
 	    yyHaveRel++;
 	}
 break;
 case 9:
-#line 143 "getdate.y"
+#line 147 "getdate.y"
 {
 	    yyHour = yyvsp[-1].Number;
 	    yyMinutes = 0;
@@ -1260,7 +1264,7 @@ case 9:
 	}
 break;
 case 10:
-#line 149 "getdate.y"
+#line 153 "getdate.y"
 {
 	    yyHour = yyvsp[-3].Number;
 	    yyMinutes = yyvsp[-1].Number;
@@ -1269,7 +1273,7 @@ case 10:
 	}
 break;
 case 11:
-#line 155 "getdate.y"
+#line 159 "getdate.y"
 {
 	    yyHour = yyvsp[-3].Number;
 	    yyMinutes = yyvsp[-1].Number;
@@ -1279,7 +1283,7 @@ case 11:
 	}
 break;
 case 12:
-#line 162 "getdate.y"
+#line 166 "getdate.y"
 {
 	    yyHour = yyvsp[-5].Number;
 	    yyMinutes = yyvsp[-3].Number;
@@ -1288,7 +1292,7 @@ case 12:
 	}
 break;
 case 13:
-#line 168 "getdate.y"
+#line 172 "getdate.y"
 {
 	    yyHour = yyvsp[-5].Number;
 	    yyMinutes = yyvsp[-3].Number;
@@ -1299,56 +1303,56 @@ case 13:
 	}
 break;
 case 14:
-#line 178 "getdate.y"
+#line 182 "getdate.y"
 {
 	    yyTimezone = yyvsp[0].Number;
 	    yyDSTmode = DSToff;
 	}
 break;
 case 15:
-#line 182 "getdate.y"
+#line 186 "getdate.y"
 {
 	    yyTimezone = yyvsp[0].Number;
 	    yyDSTmode = DSTon;
 	}
 break;
 case 16:
-#line 187 "getdate.y"
+#line 191 "getdate.y"
 {
 	    yyTimezone = yyvsp[-1].Number;
 	    yyDSTmode = DSTon;
 	}
 break;
 case 17:
-#line 193 "getdate.y"
+#line 197 "getdate.y"
 {
 	    yyDayOrdinal = 1;
 	    yyDayNumber = yyvsp[0].Number;
 	}
 break;
 case 18:
-#line 197 "getdate.y"
+#line 201 "getdate.y"
 {
 	    yyDayOrdinal = 1;
 	    yyDayNumber = yyvsp[-1].Number;
 	}
 break;
 case 19:
-#line 201 "getdate.y"
+#line 205 "getdate.y"
 {
 	    yyDayOrdinal = yyvsp[-1].Number;
 	    yyDayNumber = yyvsp[0].Number;
 	}
 break;
 case 20:
-#line 207 "getdate.y"
+#line 211 "getdate.y"
 {
 	    yyMonth = yyvsp[-2].Number;
 	    yyDay = yyvsp[0].Number;
 	}
 break;
 case 21:
-#line 211 "getdate.y"
+#line 215 "getdate.y"
 {
 	    if (yyvsp[-4].Number >= 100) {
 		yyYear = yyvsp[-4].Number;
@@ -1362,7 +1366,7 @@ case 21:
 	}
 break;
 case 22:
-#line 222 "getdate.y"
+#line 226 "getdate.y"
 {
 	    /* ISO 8601 format.  yyyy-mm-dd.  */
 	    yyYear = yyvsp[-2].Number;
@@ -1371,7 +1375,7 @@ case 22:
 	}
 break;
 case 23:
-#line 228 "getdate.y"
+#line 232 "getdate.y"
 {
 	    /* e.g. 17-JUN-1992.  */
 	    yyDay = yyvsp[-2].Number;
@@ -1380,14 +1384,14 @@ case 23:
 	}
 break;
 case 24:
-#line 234 "getdate.y"
+#line 238 "getdate.y"
 {
 	    yyMonth = yyvsp[-1].Number;
 	    yyDay = yyvsp[0].Number;
 	}
 break;
 case 25:
-#line 238 "getdate.y"
+#line 242 "getdate.y"
 {
 	    yyMonth = yyvsp[-3].Number;
 	    yyDay = yyvsp[-2].Number;
@@ -1395,14 +1399,14 @@ case 25:
 	}
 break;
 case 26:
-#line 243 "getdate.y"
+#line 247 "getdate.y"
 {
 	    yyMonth = yyvsp[0].Number;
 	    yyDay = yyvsp[-1].Number;
 	}
 break;
 case 27:
-#line 247 "getdate.y"
+#line 251 "getdate.y"
 {
 	    yyMonth = yyvsp[-1].Number;
 	    yyDay = yyvsp[-2].Number;
@@ -1410,68 +1414,68 @@ case 27:
 	}
 break;
 case 28:
-#line 254 "getdate.y"
+#line 258 "getdate.y"
 {
 	    yyRelSeconds = -yyRelSeconds;
 	    yyRelMonth = -yyRelMonth;
 	}
 break;
 case 30:
-#line 261 "getdate.y"
+#line 265 "getdate.y"
 {
 	    yyRelSeconds += yyvsp[-1].Number * yyvsp[0].Number * 60L;
 	}
 break;
 case 31:
-#line 264 "getdate.y"
+#line 268 "getdate.y"
 {
 	    yyRelSeconds += yyvsp[-1].Number * yyvsp[0].Number * 60L;
 	}
 break;
 case 32:
-#line 267 "getdate.y"
+#line 271 "getdate.y"
 {
 	    yyRelSeconds += yyvsp[0].Number * 60L;
 	}
 break;
 case 33:
-#line 270 "getdate.y"
+#line 274 "getdate.y"
 {
 	    yyRelSeconds += yyvsp[-1].Number;
 	}
 break;
 case 34:
-#line 273 "getdate.y"
+#line 277 "getdate.y"
 {
 	    yyRelSeconds += yyvsp[-1].Number;
 	}
 break;
 case 35:
-#line 276 "getdate.y"
+#line 280 "getdate.y"
 {
 	    yyRelSeconds++;
 	}
 break;
 case 36:
-#line 279 "getdate.y"
+#line 283 "getdate.y"
 {
 	    yyRelMonth += yyvsp[-1].Number * yyvsp[0].Number;
 	}
 break;
 case 37:
-#line 282 "getdate.y"
+#line 286 "getdate.y"
 {
 	    yyRelMonth += yyvsp[-1].Number * yyvsp[0].Number;
 	}
 break;
 case 38:
-#line 285 "getdate.y"
+#line 289 "getdate.y"
 {
 	    yyRelMonth += yyvsp[0].Number;
 	}
 break;
 case 39:
-#line 290 "getdate.y"
+#line 294 "getdate.y"
 {
 	    if (yyHaveTime && yyHaveDate && !yyHaveRel)
 		yyYear = yyvsp[0].Number;
@@ -1499,18 +1503,18 @@ case 39:
 	}
 break;
 case 40:
-#line 317 "getdate.y"
+#line 321 "getdate.y"
 {
 	    yyval.Meridian = MER24;
 	}
 break;
 case 41:
-#line 320 "getdate.y"
+#line 324 "getdate.y"
 {
 	    yyval.Meridian = yyvsp[0].Meridian;
 	}
 break;
-#line 1461 "getdate.c"
+#line 1465 "getdate.c"
     }
     yyssp -= yym;
     yystate = *yyssp;
