@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1996, 1998-2005, 2007-2013
+ * Copyright (c) 1996, 1998-2005, 2007-2014
  *	Todd C. Miller <Todd.Miller@courtesan.com>
  *
  * Permission to use, copy, modify, and distribute this software for any
@@ -116,7 +116,7 @@ sudo_make_pwitem(uid_t uid, const char *name)
 	total += strlen(name) + 1;
 
     /* Allocate space for struct item, struct passwd and the strings. */
-    pwitem = ecalloc(1, total);
+    pwitem = sudo_ecalloc(1, total);
     newpw = &pwitem->pw;
 
     /*
@@ -183,7 +183,7 @@ sudo_make_gritem(gid_t gid, const char *name)
     if (name != NULL)
 	total += strlen(name) + 1;
 
-    gritem = ecalloc(1, total);
+    gritem = sudo_ecalloc(1, total);
 
     /*
      * Copy in group contents and make strings relative to space
@@ -245,7 +245,7 @@ sudo_make_grlist_item(const struct passwd *pw, char * const *unused1,
     } else {
 	if (sudo_user.max_groups > 0) {
 	    ngids = sudo_user.max_groups;
-	    gids = emalloc2(ngids, sizeof(GETGROUPS_T));
+	    gids = sudo_emallocarray(ngids, sizeof(GETGROUPS_T));
 	    (void)getgrouplist(pw->pw_name, pw->pw_gid, gids, &ngids);
 	} else {
 #if defined(HAVE_SYSCONF) && defined(_SC_NGROUPS_MAX)
@@ -253,17 +253,17 @@ sudo_make_grlist_item(const struct passwd *pw, char * const *unused1,
 	    if (ngids < 0)
 #endif
 		ngids = NGROUPS_MAX * 2;
-	    gids = emalloc2(ngids, sizeof(GETGROUPS_T));
+	    gids = sudo_emallocarray(ngids, sizeof(GETGROUPS_T));
 	    if (getgrouplist(pw->pw_name, pw->pw_gid, gids, &ngids) == -1) {
-		efree(gids);
-		gids = emalloc2(ngids, sizeof(GETGROUPS_T));
+		sudo_efree(gids);
+		gids = sudo_emallocarray(ngids, sizeof(GETGROUPS_T));
 		if (getgrouplist(pw->pw_name, pw->pw_gid, gids, &ngids) == -1)
 		    ngids = -1;
 	    }
 	}
     }
     if (ngids <= 0) {
-	efree(gids);
+	sudo_efree(gids);
 	debug_return_ptr(NULL);
     }
 
@@ -285,7 +285,7 @@ sudo_make_grlist_item(const struct passwd *pw, char * const *unused1,
     total += groupname_len * ngids;
 
 again:
-    grlitem = ecalloc(1, total);
+    grlitem = sudo_ecalloc(1, total);
 
     /*
      * Copy in group list and make pointers relative to space
@@ -322,7 +322,7 @@ again:
 	    len = strlen(grp->gr_name) + 1;
 	    if (cp - (char *)grlitem + len > total) {
 		total += len + groupname_len;
-		efree(grlitem);
+		sudo_efree(grlitem);
 		sudo_gr_delref(grp);
 		goto again;
 	    }
@@ -333,7 +333,7 @@ again:
 	}
     }
     grlist->ngroups = ngroups;
-    efree(gids);
+    sudo_efree(gids);
 
 #ifdef HAVE_SETAUTHDB
     aix_restoreauthdb();
