@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2009-2013 Todd C. Miller <Todd.Miller@courtesan.com>
+ * Copyright (c) 2009-2014 Todd C. Miller <Todd.Miller@courtesan.com>
  *
  * Permission to use, copy, modify, and distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
@@ -81,28 +81,29 @@
 #else
 # include "compat/stdbool.h"
 #endif /* HAVE_STDBOOL_H */
-#ifdef HAVE_GETOPT_LONG
-# include <getopt.h>
-# else
-# include "compat/getopt.h"
-#endif /* HAVE_GETOPT_LONG */
 
 #include <pathnames.h>
 
-#include "gettext.h"		/* must be included before missing.h */
+#include "sudo_gettext.h"	/* must be included before sudo_compat.h */
 
-#include "missing.h"
-#include "alloc.h"
-#include "fatal.h"
+#include "sudo_compat.h"
+#include "sudo_alloc.h"
+#include "sudo_fatal.h"
 #include "logging.h"
 #include "iolog.h"
-#include "queue.h"
+#include "sudo_queue.h"
 #include "sudo_plugin.h"
 #include "sudo_conf.h"
 #include "sudo_debug.h"
 #include "sudo_event.h"
 #include "sudo_util.h"
 
+#ifdef HAVE_GETOPT_LONG
+# include <getopt.h>
+# else
+# include "compat/getopt.h"
+#endif /* HAVE_GETOPT_LONG */
+
 #ifndef LINE_MAX
 # define LINE_MAX 2048
 #endif
@@ -251,7 +252,7 @@ main(int argc, char *argv[])
     textdomain("sudoers");
 
     /* Register fatal/fatalx callback. */
-    fatal_callback_register(sudoreplay_cleanup);
+    sudo_fatal_callback_register(sudoreplay_cleanup);
 
     /* Read sudo.conf. */
     sudo_conf_read(NULL);
@@ -272,7 +273,7 @@ main(int argc, char *argv[])
 		else if (strcmp(cp, "ttyout") == 0)
 		    io_log_files[IOFD_TTYOUT].enabled = true;
 		else
-		    fatalx(U_("invalid filter option: %s"), optarg);
+		    sudo_fatalx(U_("invalid filter option: %s"), optarg);
 	    }
 	    break;
 	case 'h':
@@ -285,13 +286,13 @@ main(int argc, char *argv[])
 	    errno = 0;
 	    max_wait = strtod(optarg, &ep);
 	    if (*ep != '\0' || errno != 0)
-		fatalx(U_("invalid max wait: %s"), optarg);
+		sudo_fatalx(U_("invalid max wait: %s"), optarg);
 	    break;
 	case 's':
 	    errno = 0;
 	    speed_factor = strtod(optarg, &ep);
 	    if (*ep != '\0' || errno != 0)
-		fatalx(U_("invalid speed factor: %s"), optarg);
+		sudo_fatalx(U_("invalid speed factor: %s"), optarg);
 	    break;
 	case 'V':
 	    (void) printf(_("%s version %s\n"), getprogname(), PACKAGE_VERSION);
@@ -326,13 +327,13 @@ main(int argc, char *argv[])
 	plen = snprintf(path, sizeof(path), "%s/%.2s/%.2s/%.2s/timing",
 	    session_dir, id, &id[2], &id[4]);
 	if (plen <= 0 || (size_t)plen >= sizeof(path))
-	    fatalx(U_("%s/%.2s/%.2s/%.2s/timing: %s"), session_dir,
+	    sudo_fatalx(U_("%s/%.2s/%.2s/%.2s/timing: %s"), session_dir,
 		id, &id[2], &id[4], strerror(ENAMETOOLONG));
     } else {
 	plen = snprintf(path, sizeof(path), "%s/%s/timing",
 	    session_dir, id);
 	if (plen <= 0 || (size_t)plen >= sizeof(path))
-	    fatalx(U_("%s/%s/timing: %s"), session_dir,
+	    sudo_fatalx(U_("%s/%s/timing: %s"), session_dir,
 		id, strerror(ENAMETOOLONG));
     }
     plen -= 7;
@@ -340,7 +341,7 @@ main(int argc, char *argv[])
     /* Open files for replay, applying replay filter for the -f flag. */
     for (idx = 0; idx < IOFD_MAX; idx++) {
 	if (open_io_fd(path, plen, &io_log_files[idx]) == -1) 
-	    fatal(U_("unable to open %s"), path);
+	    sudo_fatal(U_("unable to open %s"), path);
     }
 
     /* Parse log file. */
@@ -351,7 +352,7 @@ main(int argc, char *argv[])
     printf(_("Replaying sudo session: %s\n"), li->cmd);
 
     /* Make sure the terminal is large enough. */
-    get_ttysize(&rows, &cols);
+    sudo_get_ttysize(&rows, &cols);
     if (li->rows != 0 && li->cols != 0) {
 	if (li->rows > rows) {
 	    printf(_("Warning: your terminal is too small to properly replay the log.\n"));
@@ -366,7 +367,7 @@ main(int argc, char *argv[])
     /* Replay session corresponding to io_log_files[]. */
     replay_session(max_wait, decimal);
 
-    term_restore(STDIN_FILENO, 1);
+    sudo_term_restore(STDIN_FILENO, 1);
 done:
     sudo_debug_exit_int(__func__, __FILE__, __LINE__, sudo_debug_subsys, exitcode);
     exit(exitcode);
@@ -409,21 +410,21 @@ replay_session(const double max_wait, const char *decimal)
 	idx = fcntl(STDIN_FILENO, F_GETFL, 0);
 	if (idx != -1)
 	    (void) fcntl(STDIN_FILENO, F_SETFL, idx | O_NONBLOCK);
-	if (!term_raw(STDIN_FILENO, 1))
-	    fatal(U_("unable to set tty to raw mode"));
+	if (!sudo_term_raw(STDIN_FILENO, 1))
+	    sudo_fatal(U_("unable to set tty to raw mode"));
     }
 
     /* Setup event base and input/output events. */
     evbase = sudo_ev_base_alloc();
     if (evbase == NULL)
-	fatal(NULL);
+	sudo_fatal(NULL);
     input_ev = sudo_ev_alloc(STDIN_FILENO, interactive ? SUDO_EV_READ :
 	SUDO_EV_TIMEOUT, check_input, sudo_ev_self_cbarg());
     if (input_ev == NULL)
-        fatal(NULL);
+        sudo_fatal(NULL);
     output_ev = sudo_ev_alloc(STDIN_FILENO, SUDO_EV_WRITE, write_output, &wc);
     if (output_ev == NULL)
-        fatal(NULL);
+        sudo_fatal(NULL);
 
     /*
      * Read each line of the timing file, displaying the output streams.
@@ -441,7 +442,7 @@ replay_session(const double max_wait, const char *decimal)
 
 	buf[strcspn(buf, "\n")] = '\0';
 	if (!parse_timing(buf, decimal, &idx, &seconds, &nbytes))
-	    fatalx(U_("invalid timing file line: %s"), buf);
+	    sudo_fatalx(U_("invalid timing file line: %s"), buf);
 
 	/* Adjust delay using speed factor and clamp to max_wait */
 	to_wait = seconds / speed_factor;
@@ -502,8 +503,8 @@ replay_session(const double max_wait, const char *decimal)
 		    /* Store the line in iov followed by \r\n pair. */
 		    if (iovcnt + 3 > iovmax) {
 			iov = iovmax ?
-			    erealloc3(iov, iovmax <<= 1, sizeof(*iov)) :
-			    emalloc2(iovmax = 32, sizeof(*iov));
+			    sudo_ereallocarray(iov, iovmax <<= 1, sizeof(*iov)) :
+			    sudo_emallocarray(iovmax = 32, sizeof(*iov));
 		    }
 		    linelen = (size_t)(ep - cp) + 1;
 		    iov[iovcnt].iov_base = cp;
@@ -580,7 +581,7 @@ write_output(int fd, int what, void *v)
     switch (nwritten) {
     case -1:
 	if (errno != EINTR && errno != EAGAIN)
-	    fatal(U_("unable to write to %s"), "stdout");
+	    sudo_fatal(U_("unable to write to %s"), "stdout");
 	break;
     case 0:
 	break;
@@ -647,7 +648,7 @@ parse_expr(struct search_node_list *head, char *argv[], bool sub_expr)
 	    continue;
 	case 'c': /* command */
 	    if (av[0][1] == '\0')
-		fatalx(U_("ambiguous expression \"%s\""), *av);
+		sudo_fatalx(U_("ambiguous expression \"%s\""), *av);
 	    if (strncmp(*av, "cwd", strlen(*av)) == 0)
 		type = ST_CWD;
 	    else if (strncmp(*av, "command", strlen(*av)) == 0)
@@ -672,7 +673,7 @@ parse_expr(struct search_node_list *head, char *argv[], bool sub_expr)
 	    break;
 	case 't': /* tty or to date */
 	    if (av[0][1] == '\0')
-		fatalx(U_("ambiguous expression \"%s\""), *av);
+		sudo_fatalx(U_("ambiguous expression \"%s\""), *av);
 	    if (strncmp(*av, "todate", strlen(*av)) == 0)
 		type = ST_TODATE;
 	    else if (strncmp(*av, "tty", strlen(*av)) == 0)
@@ -694,16 +695,16 @@ parse_expr(struct search_node_list *head, char *argv[], bool sub_expr)
 	    if (av[0][1] != '\0')
 		goto bad;
 	    if (!sub_expr)
-		fatalx(U_("unmatched ')' in expression"));
+		sudo_fatalx(U_("unmatched ')' in expression"));
 	    debug_return_int(av - argv + 1);
 	bad:
 	default:
-	    fatalx(U_("unknown search term \"%s\""), *av);
+	    sudo_fatalx(U_("unknown search term \"%s\""), *av);
 	    /* NOTREACHED */
 	}
 
 	/* Allocate new search node */
-	sn = ecalloc(1, sizeof(*sn));
+	sn = sudo_ecalloc(1, sizeof(*sn));
 	sn->type = type;
 	sn->or = or;
 	sn->negated = not;
@@ -712,17 +713,17 @@ parse_expr(struct search_node_list *head, char *argv[], bool sub_expr)
 	    av += parse_expr(&sn->u.expr, av + 1, true);
 	} else {
 	    if (*(++av) == NULL)
-		fatalx(U_("%s requires an argument"), av[-1]);
+		sudo_fatalx(U_("%s requires an argument"), av[-1]);
 #ifdef HAVE_REGCOMP
 	    if (type == ST_PATTERN) {
 		if (regcomp(&sn->u.cmdre, *av, REG_EXTENDED|REG_NOSUB) != 0)
-		    fatalx(U_("invalid regular expression: %s"), *av);
+		    sudo_fatalx(U_("invalid regular expression: %s"), *av);
 	    } else
 #endif
 	    if (type == ST_TODATE || type == ST_FROMDATE) {
 		sn->u.tstamp = get_date(*av);
 		if (sn->u.tstamp == -1)
-		    fatalx(U_("could not parse date \"%s\""), *av);
+		    sudo_fatalx(U_("could not parse date \"%s\""), *av);
 	    } else {
 		sn->u.ptr = *av;
 	    }
@@ -731,11 +732,11 @@ parse_expr(struct search_node_list *head, char *argv[], bool sub_expr)
 	STAILQ_INSERT_TAIL(head, sn, entries);
     }
     if (sub_expr)
-	fatalx(U_("unmatched '(' in expression"));
+	sudo_fatalx(U_("unmatched '(' in expression"));
     if (or)
-	fatalx(U_("illegal trailing \"or\""));
+	sudo_fatalx(U_("illegal trailing \"or\""));
     if (not)
-	fatalx(U_("illegal trailing \"!\""));
+	sudo_fatalx(U_("illegal trailing \"!\""));
 
     debug_return_int(av - argv);
 }
@@ -774,7 +775,7 @@ match_expr(struct search_node_list *head, struct log_info *log, bool last_match)
 	    if (rc && rc != REG_NOMATCH) {
 		char buf[BUFSIZ];
 		regerror(rc, &sn->u.cmdre, buf, sizeof(buf));
-		fatalx("%s", buf);
+		sudo_fatalx("%s", buf);
 	    }
 	    res = rc == REG_NOMATCH ? 0 : 1;
 #else
@@ -788,7 +789,7 @@ match_expr(struct search_node_list *head, struct log_info *log, bool last_match)
 	    res = log->tstamp <= sn->u.tstamp;
 	    break;
 	default:
-	    fatalx(U_("unknown search type %d"), sn->type);
+	    sudo_fatalx(U_("unknown search type %d"), sn->type);
 	    /* NOTREACHED */
 	}
 	if (sn->negated)
@@ -811,7 +812,7 @@ parse_logfile(char *logfile)
 
     fp = fopen(logfile, "r");
     if (fp == NULL) {
-	warning(U_("unable to open %s"), logfile);
+	sudo_warn(U_("unable to open %s"), logfile);
 	goto bad;
     }
 
@@ -821,11 +822,11 @@ parse_logfile(char *logfile)
      *  2) cwd
      *  3) command with args
      */
-    li = ecalloc(1, sizeof(*li));
+    li = sudo_ecalloc(1, sizeof(*li));
     if (getline(&buf, &bufsize, fp) == -1 ||
 	getline(&li->cwd, &cwdsize, fp) == -1 ||
 	getline(&li->cmd, &cmdsize, fp) == -1) {
-	warning(U_("%s: invalid log file"), logfile);
+	sudo_warn(U_("%s: invalid log file"), logfile);
 	goto bad;
     }
 
@@ -843,50 +844,50 @@ parse_logfile(char *logfile)
 
     /* timestamp */
     if ((ep = strchr(cp, ':')) == NULL) {
-	warning(U_("%s: time stamp field is missing"), logfile);
+	sudo_warn(U_("%s: time stamp field is missing"), logfile);
 	goto bad;
     }
     *ep = '\0';
     li->tstamp = sizeof(time_t) == 4 ? strtonum(cp, INT_MIN, INT_MAX, &errstr) :
 	strtonum(cp, LLONG_MIN, LLONG_MAX, &errstr);
     if (errstr != NULL) {
-	warning(U_("%s: time stamp %s: %s"), logfile, cp, errstr);
+	sudo_warn(U_("%s: time stamp %s: %s"), logfile, cp, errstr);
 	goto bad;
     }
 
     /* user */
     cp = ep + 1;
     if ((ep = strchr(cp, ':')) == NULL) {
-	warning(U_("%s: user field is missing"), logfile);
+	sudo_warn(U_("%s: user field is missing"), logfile);
 	goto bad;
     }
-    li->user = estrndup(cp, (size_t)(ep - cp));
+    li->user = sudo_estrndup(cp, (size_t)(ep - cp));
 
     /* runas user */
     cp = ep + 1;
     if ((ep = strchr(cp, ':')) == NULL) {
-	warning(U_("%s: runas user field is missing"), logfile);
+	sudo_warn(U_("%s: runas user field is missing"), logfile);
 	goto bad;
     }
-    li->runas_user = estrndup(cp, (size_t)(ep - cp));
+    li->runas_user = sudo_estrndup(cp, (size_t)(ep - cp));
 
     /* runas group */
     cp = ep + 1;
     if ((ep = strchr(cp, ':')) == NULL) {
-	warning(U_("%s: runas group field is missing"), logfile);
+	sudo_warn(U_("%s: runas group field is missing"), logfile);
 	goto bad;
     }
     if (cp != ep)
-	li->runas_group = estrndup(cp, (size_t)(ep - cp));
+	li->runas_group = sudo_estrndup(cp, (size_t)(ep - cp));
 
     /* tty, followed by optional rows + columns */
     cp = ep + 1;
     if ((ep = strchr(cp, ':')) == NULL) {
 	/* just the tty */
-	li->tty = estrdup(cp);
+	li->tty = sudo_estrdup(cp);
     } else {
 	/* tty followed by rows + columns */
-	li->tty = estrndup(cp, (size_t)(ep - cp));
+	li->tty = sudo_estrndup(cp, (size_t)(ep - cp));
 	cp = ep + 1;
 	/* need to NULL out separator to use strtonum() */
 	if ((ep = strchr(cp, ':')) != NULL) {
@@ -907,13 +908,13 @@ parse_logfile(char *logfile)
 	}
     }
     fclose(fp);
-    efree(buf);
+    sudo_efree(buf);
     debug_return_ptr(li);
 
 bad:
     if (fp != NULL)
 	fclose(fp);
-    efree(buf);
+    sudo_efree(buf);
     free_log_info(li);
     debug_return_ptr(NULL);
 }
@@ -922,13 +923,13 @@ static void
 free_log_info(struct log_info *li)
 {
     if (li != NULL) {
-	efree(li->cwd);
-	efree(li->user);
-	efree(li->runas_user);
-	efree(li->runas_group);
-	efree(li->tty);
-	efree(li->cmd);
-	efree(li);
+	sudo_efree(li->cwd);
+	sudo_efree(li->user);
+	sudo_efree(li->runas_user);
+	sudo_efree(li->runas_group);
+	sudo_efree(li->tty);
+	sudo_efree(li->cmd);
+	sudo_efree(li);
     }
 }
 
@@ -988,7 +989,7 @@ session_compare(const void *v1, const void *v2)
     return strcmp(s1, s2);
 }
 
-/* XXX - always returns 0, calls fatal() on failure */
+/* XXX - always returns 0, calls sudo_fatal() on failure */
 static int
 find_sessions(const char *dir, REGEX_T *re, const char *user, const char *tty)
 {
@@ -1008,19 +1009,19 @@ find_sessions(const char *dir, REGEX_T *re, const char *user, const char *tty)
 
     d = opendir(dir);
     if (d == NULL)
-	fatal(U_("unable to open %s"), dir);
+	sudo_fatal(U_("unable to open %s"), dir);
 
     /* XXX - would be faster to chdir and use relative names */
     sdlen = strlcpy(pathbuf, dir, sizeof(pathbuf));
     if (sdlen + 1 >= sizeof(pathbuf)) {
 	errno = ENAMETOOLONG;
-	fatal("%s/", dir);
+	sudo_fatal("%s/", dir);
     }
     pathbuf[sdlen++] = '/';
     pathbuf[sdlen] = '\0';
 
     /* Store potential session dirs for sorting. */
-    sessions = emalloc2(sessions_size, sizeof(char *));
+    sessions = sudo_emallocarray(sessions_size, sizeof(char *));
     while ((dp = readdir(d)) != NULL) {
 	/* Skip "." and ".." */
 	if (dp->d_name[0] == '.' && (dp->d_name[1] == '\0' ||
@@ -1040,9 +1041,9 @@ find_sessions(const char *dir, REGEX_T *re, const char *user, const char *tty)
 	/* Add name to session list. */
 	if (sessions_len + 1 > sessions_size) {
 	    sessions_size <<= 1;
-	    sessions = erealloc3(sessions, sessions_size, sizeof(char *));
+	    sessions = sudo_ereallocarray(sessions, sessions_size, sizeof(char *));
 	}
-	sessions[sessions_len++] = estrdup(dp->d_name);
+	sessions[sessions_len++] = sudo_estrdup(dp->d_name);
     }
     closedir(d);
 
@@ -1053,9 +1054,9 @@ find_sessions(const char *dir, REGEX_T *re, const char *user, const char *tty)
 	    "%s/log", sessions[i]);
 	if (len <= 0 || (size_t)len >= sizeof(pathbuf) - sdlen) {
 	    errno = ENAMETOOLONG;
-	    fatal("%s/%s/log", dir, sessions[i]);
+	    sudo_fatal("%s/%s/log", dir, sessions[i]);
 	}
-	efree(sessions[i]);
+	sudo_efree(sessions[i]);
 
 	/* Check for dir with a log file. */
 	if (lstat(pathbuf, &sb) == 0 && S_ISREG(sb.st_mode)) {
@@ -1067,12 +1068,12 @@ find_sessions(const char *dir, REGEX_T *re, const char *user, const char *tty)
 		find_sessions(pathbuf, re, user, tty);
 	}
     }
-    efree(sessions);
+    sudo_efree(sessions);
 
     debug_return_int(0);
 }
 
-/* XXX - always returns 0, calls fatal() on failure */
+/* XXX - always returns 0, calls sudo_fatal() on failure */
 static int
 list_sessions(int argc, char **argv, const char *pattern, const char *user,
     const char *tty)
@@ -1088,7 +1089,7 @@ list_sessions(int argc, char **argv, const char *pattern, const char *user,
     if (pattern) {
 	re = &rebuf;
 	if (regcomp(re, pattern, REG_EXTENDED|REG_NOSUB) != 0)
-	    fatalx(U_("invalid regular expression: %s"), pattern);
+	    sudo_fatalx(U_("invalid regular expression: %s"), pattern);
     }
 #else
     re = (char *) pattern;
@@ -1115,7 +1116,7 @@ check_input(int fd, int what, void *v)
 	switch (read(fd, &ch, 1)) {
 	case -1:
 	    if (errno != EINTR && errno != EAGAIN)
-		fatal(U_("unable to read %s"), "stdin");
+		sudo_fatal(U_("unable to read %s"), "stdin");
 	    break;
 	case 0:
 	    /* Ignore EOF. */
@@ -1254,12 +1255,12 @@ help(void)
 }
 
 /*
- * Cleanup hook for fatal()/fatalx()
+ * Cleanup hook for sudo_fatal()/sudo_fatalx()
   */
 static void
 sudoreplay_cleanup(void)
 {
-    term_restore(STDIN_FILENO, 0);
+    sudo_term_restore(STDIN_FILENO, 0);
 }
 
 /*
@@ -1269,6 +1270,6 @@ sudoreplay_cleanup(void)
 static void
 sudoreplay_handler(int signo)
 {
-    term_restore(STDIN_FILENO, 0);
+    sudo_term_restore(STDIN_FILENO, 0);
     kill(getpid(), signo);
 }
