@@ -119,23 +119,25 @@ static int fork_cmnd(struct command_details *details, int sv[2])
 #else
     sa.sa_handler = handler;
 #endif
-    sudo_sigaction(SIGCONT, &sa, NULL);
+    if (sudo_sigaction(SIGCONT, &sa, NULL) != 0)
+	sudo_warn(U_("unable to set handler for signal %d"), SIGCONT);
 #ifdef SA_SIGINFO
     sa.sa_sigaction = handler_user_only;
 #endif
-    sudo_sigaction(SIGTSTP, &sa, NULL);
+    if (sudo_sigaction(SIGTSTP, &sa, NULL) != 0)
+	sudo_warn(U_("unable to set handler for signal %d"), SIGTSTP);
 
     /*
      * The policy plugin's session init must be run before we fork
      * or certain pam modules won't be able to track their state.
      */
     if (policy_init_session(details) != true)
-	fatalx(U_("policy plugin failed session initialization"));
+	sudo_fatalx(U_("policy plugin failed session initialization"));
 
     cmnd_pid = sudo_debug_fork();
     switch (cmnd_pid) {
     case -1:
-	fatal(U_("unable to fork"));
+	sudo_fatal(U_("unable to fork"));
 	break;
     case 0:
 	/* child */
@@ -230,11 +232,14 @@ backchannel_cb(int fd, int what, void *v)
 	    /* Short read or EOF. */
 	    sudo_debug_printf(SUDO_DEBUG_ERROR,
 		"failed to read child status: %s", n ? "short read" : "EOF");
-	    /*
-	     * If not logging I/O we may get EOF when the command is
-	     * executed and sv is closed.  It is safe to ignore this.
-	     */
-	    if (ec->log_io || n != 0) {
+	    if (!ec->log_io && n == 0) {
+		/*
+		 * If not logging I/O we may get EOF when the command is
+		 * executed and the other end of the backchannel is closed.
+		 * Just remove the event in this case.
+		 */
+		sudo_ev_del(ec->evbase, backchannel_event);
+	    } else {
 		/* XXX - need new CMD_ type for monitor errors. */
 		errno = n ? EIO : ECONNRESET;
 		ec->cstat->type = CMD_ERRNO;
@@ -297,29 +302,29 @@ exec_event_setup(int backchannel, struct exec_closure *ec)
 
     evbase = sudo_ev_base_alloc();
     if (evbase == NULL)
-	fatal(NULL);
+	sudo_fatal(NULL);
 
     /* Event for incoming signals via signal_pipe. */
     signal_event = sudo_ev_alloc(signal_pipe[0],
 	SUDO_EV_READ|SUDO_EV_PERSIST, signal_pipe_cb, ec);
     if (signal_event == NULL)
-	fatal(NULL);
+	sudo_fatal(NULL);
     if (sudo_ev_add(evbase, signal_event, NULL, false) == -1)
-	fatal(U_("unable to add event to queue"));
+	sudo_fatal(U_("unable to add event to queue"));
 
     /* Event for command status via backchannel. */
     backchannel_event = sudo_ev_alloc(backchannel,
 	SUDO_EV_READ|SUDO_EV_PERSIST, backchannel_cb, ec);
     if (backchannel_event == NULL)
-	fatal(NULL);
+	sudo_fatal(NULL);
     if (sudo_ev_add(evbase, backchannel_event, NULL, false) == -1)
-	fatal(U_("unable to add event to queue"));
+	sudo_fatal(U_("unable to add event to queue"));
 
     /* The signal forwarding event gets added on demand. */
     sigfwd_event = sudo_ev_alloc(backchannel,
 	SUDO_EV_WRITE, forward_signals, NULL);
     if (sigfwd_event == NULL)
-	fatal(NULL);
+	sudo_fatal(NULL);
 
     sudo_debug_printf(SUDO_DEBUG_INFO, "signal pipe fd %d\n", signal_pipe[0]);
     sudo_debug_printf(SUDO_DEBUG_INFO, "backchannel fd %d\n", backchannel);
@@ -393,7 +398,7 @@ sudo_execute(struct command_details *details, struct command_status *cstat)
      * Parent sends signal info to child and child sends back wait status.
      */
     if (socketpair(PF_UNIX, SOCK_STREAM, 0, sv) == -1)
-	fatal(U_("unable to create sockets"));
+	sudo_fatal(U_("unable to create sockets"));
 
     /*
      * Signals to forward to the child process (excluding SIGALRM and SIGCHLD).
@@ -409,12 +414,22 @@ sudo_execute(struct command_details *details, struct command_status *cstat)
 #else
     sa.sa_handler = handler;
 #endif
-    sudo_sigaction(SIGTERM, &sa, NULL);
-    sudo_sigaction(SIGALRM, &sa, NULL); /* XXX - only if there is a timeout */
-    sudo_sigaction(SIGCHLD, &sa, NULL);
-    sudo_sigaction(SIGPIPE, &sa, NULL);
-    sudo_sigaction(SIGUSR1, &sa, NULL);
-    sudo_sigaction(SIGUSR2, &sa, NULL);
+    if (sudo_sigaction(SIGTERM, &sa, NULL) != 0)
+	sudo_warn(U_("unable to set handler for signal %d"), SIGTERM);
+    if (sudo_sigaction(SIGALRM, &sa, NULL) != 0)
+	sudo_warn(U_("unable to set handler for signal %d"), SIGALRM);
+    if (sudo_sigaction(SIGCHLD, &sa, NULL) != 0)
+	sudo_warn(U_("unable to set handler for signal %d"), SIGCHLD);
+    if (sudo_sigaction(SIGPIPE, &sa, NULL) != 0)
+	sudo_warn(U_("unable to set handler for signal %d"), SIGPIPE);
+    if (sudo_sigaction(SIGUSR1, &sa, NULL) != 0)
+	sudo_warn(U_("unable to set handler for signal %d"), SIGUSR1);
+    if (sudo_sigaction(SIGUSR2, &sa, NULL) != 0)
+	sudo_warn(U_("unable to set handler for signal %d"), SIGUSR2);
+#ifdef SIGINFO
+    if (sudo_sigaction(SIGINFO, &sa, NULL) != 0)
+	sudo_warn(U_("unable to set handler for signal %d"), SIGINFO);
+#endif
 
     /*
      * When not running the command in a pty, we do not want to
@@ -429,9 +444,12 @@ sudo_execute(struct command_details *details, struct command_status *cstat)
 	sa.sa_sigaction = handler_user_only;
     }
 #endif
-    sudo_sigaction(SIGHUP, &sa, NULL);
-    sudo_sigaction(SIGINT, &sa, NULL);
-    sudo_sigaction(SIGQUIT, &sa, NULL);
+    if (sudo_sigaction(SIGHUP, &sa, NULL) != 0)
+	sudo_warn(U_("unable to set handler for signal %d"), SIGHUP);
+    if (sudo_sigaction(SIGINT, &sa, NULL) != 0)
+	sudo_warn(U_("unable to set handler for signal %d"), SIGINT);
+    if (sudo_sigaction(SIGQUIT, &sa, NULL) != 0)
+	sudo_warn(U_("unable to set handler for signal %d"), SIGQUIT);
 
     /*
      * Child will run the command in the pty, parent will pass data
@@ -476,10 +494,13 @@ sudo_execute(struct command_details *details, struct command_status *cstat)
     if (log_io)
 	add_io_events(evbase);
     if (sudo_ev_loop(evbase, 0) == -1)
-	warning(U_("error in event loop"));
+	sudo_warn(U_("error in event loop"));
     if (sudo_ev_got_break(evbase)) {
 	/* error from callback */
 	sudo_debug_printf(SUDO_DEBUG_ERROR, "event loop exited prematurely");
+	/* kill command if still running and not I/O logging */
+	if (!log_io && kill(child, 0) == 0)
+	    terminate_command(child, true);
     }
 
     if (log_io) {
@@ -491,7 +512,7 @@ sudo_execute(struct command_details *details, struct command_status *cstat)
     if (ISSET(details->flags, CD_RBAC_ENABLED)) {
 	/* This is probably not needed in log_io mode. */
 	if (selinux_restore_tty() != 0)
-	    warningx(U_("unable to restore tty label"));
+	    sudo_warnx(U_("unable to restore tty label"));
     }
 #endif
 
@@ -501,7 +522,7 @@ sudo_execute(struct command_details *details, struct command_status *cstat)
     sudo_ev_free(signal_event);
     sudo_ev_free(backchannel_event);
     TAILQ_FOREACH_SAFE(sigfwd, &sigfwd_list, entries, sigfwd_next) {
-	efree(sigfwd);
+	sudo_efree(sigfwd);
     }
     TAILQ_INIT(&sigfwd_list);
 done:
@@ -560,7 +581,7 @@ dispatch_signal(struct sudo_event_base *evbase, pid_t child,
 			    if (child_pgrp != ppgrp) {
 				if (tcsetpgrp(fd, child_pgrp) == 0) {
 				    if (killpg(child_pgrp, SIGCONT) != 0) {
-					warning("kill(%d, SIGCONT)",
+					sudo_warn("kill(%d, SIGCONT)",
 					    (int)child_pgrp);
 				    }
 				    close(fd);
@@ -575,12 +596,19 @@ dispatch_signal(struct sudo_event_base *evbase, pid_t child,
 		    sigemptyset(&sa.sa_mask);
 		    sa.sa_flags = SA_RESTART;
 		    sa.sa_handler = SIG_DFL;
-		    sudo_sigaction(SIGTSTP, &sa, &osa);
+		    if (sudo_sigaction(SIGTSTP, &sa, &osa) != 0) {
+			sudo_warn(U_("unable to set handler for signal %d"),
+			    SIGTSTP);
+		    }
 		}
 		if (kill(getpid(), signo) != 0)
-		    warning("kill(%d, SIG%s)", (int)getpid(), signame);
-		if (signo == SIGTSTP)
-		    sudo_sigaction(SIGTSTP, &osa, NULL);
+		    sudo_warn("kill(%d, SIG%s)", (int)getpid(), signame);
+		if (signo == SIGTSTP) {
+		    if (sudo_sigaction(SIGTSTP, &osa, NULL) != 0) {
+			sudo_warn(U_("unable to restore handler for signal %d"),
+			    SIGTSTP);
+		    }
+		}
 		if (fd != -1) {
 		    /*
 		     * Restore command's process group if different.
@@ -594,6 +622,7 @@ dispatch_signal(struct sudo_event_base *evbase, pid_t child,
 		/* Child has exited or been killed, we are done. */
 		cstat->type = CMD_WSTATUS;
 		cstat->val = status;
+		sudo_ev_del(evbase, signal_event);
 		sudo_ev_loopexit(evbase);
 		goto done;
 	    }
@@ -603,7 +632,7 @@ dispatch_signal(struct sudo_event_base *evbase, pid_t child,
 	if (signo == SIGALRM) {
 	    terminate_command(child, false);
 	} else if (kill(child, signo) != 0) {
-	    warning("kill(%d, SIG%s)", (int)child, signame);
+	    sudo_warn("kill(%d, SIG%s)", (int)child, signame);
 	}
     }
     rc = 0;
@@ -612,7 +641,7 @@ done:
 }
 
 /*
- * Forward a signal to the monitory (pty version).
+ * Forward a signal to the monitor (pty version).
  */
 static int
 dispatch_signal_pty(struct sudo_event_base *evbase, pid_t child,
@@ -692,10 +721,9 @@ signal_pipe_cb(int fd, int what, void *v)
 		continue;
 	    /* On error, store errno and break out of the event loop. */
 	    if (errno != EAGAIN) {
-		sudo_debug_printf(SUDO_DEBUG_ERROR,
-		    "error reading signal pipe %s", strerror(errno));
 		ec->cstat->type = CMD_ERRNO;
 		ec->cstat->val = errno;
+		sudo_warn(U_("error reading from signal pipe"));
 		sudo_ev_loopbreak(ec->evbase);
 	    }
 	    break;
@@ -715,7 +743,7 @@ signal_pipe_cb(int fd, int what, void *v)
 }
 
 /*
- * Drain pending signals from signale_pipe written by sudo_handler().
+ * Drain pending signals from signal_pipe written by sudo_handler().
  * Handles the case where the signal was sent to us before
  * we have executed the command.
  * Returns 1 if we should terminate, else 0.
@@ -763,9 +791,10 @@ dispatch_pending_signals(struct command_status *cstat)
 	sigemptyset(&sa.sa_mask);
 	sa.sa_flags = SA_RESTART;
 	sa.sa_handler = SIG_DFL;
-	sudo_sigaction(SIGTSTP, &sa, NULL);
+	if (sudo_sigaction(SIGTSTP, &sa, NULL) != 0)
+	    sudo_warn(U_("unable to set handler for signal %d"), SIGTSTP);
 	if (kill(getpid(), SIGTSTP) != 0)
-	    warning("kill(%d, SIGTSTP)", (int)getpid());
+	    sudo_warn("kill(%d, SIGTSTP)", (int)getpid());
 	/* No need to reinstall SIGTSTP handler. */
     }
     debug_return_int(rval);
@@ -799,7 +828,7 @@ forward_signals(int sock, int what, void *v)
 	    nsent = send(sock, &cstat, sizeof(cstat), 0);
 	} while (nsent == -1 && errno == EINTR);
 	TAILQ_REMOVE(&sigfwd_list, sigfwd, entries);
-	efree(sigfwd);
+	sudo_efree(sigfwd);
 	if (nsent != sizeof(cstat)) {
 	    if (errno == EPIPE) {
 		struct sigforward *sigfwd_next;
@@ -807,7 +836,7 @@ forward_signals(int sock, int what, void *v)
 		    "broken pipe writing to child over backchannel");
 		/* Other end of socket gone, empty out sigfwd_list. */
 		TAILQ_FOREACH_SAFE(sigfwd, &sigfwd_list, entries, sigfwd_next) {
-		    efree(sigfwd);
+		    sudo_efree(sigfwd);
 		}
 		TAILQ_INIT(&sigfwd_list);
 		/* XXX - child (monitor) is dead, we should exit too? */
@@ -835,12 +864,12 @@ schedule_signal(struct sudo_event_base *evbase, int signo)
 	snprintf(signame, sizeof(signame), "%d", signo);
     sudo_debug_printf(SUDO_DEBUG_DIAG, "scheduled SIG%s for child", signame);
 
-    sigfwd = ecalloc(1, sizeof(*sigfwd));
+    sigfwd = sudo_ecalloc(1, sizeof(*sigfwd));
     sigfwd->signo = signo;
     TAILQ_INSERT_TAIL(&sigfwd_list, sigfwd, entries);
 
     if (sudo_ev_add(evbase, sigfwd_event, NULL, true) == -1)
-	fatal(U_("unable to add event to queue"));
+	sudo_fatal(U_("unable to add event to queue"));
 
     debug_return;
 }
@@ -856,19 +885,29 @@ handler(int s, siginfo_t *info, void *context)
     unsigned char signo = (unsigned char)s;
 
     /*
-     * If the signal came from the command we ran, just ignore
-     * it since we don't want the child to indirectly kill itself.
-     * This can happen with, e.g. BSD-derived versions of reboot
-     * that call kill(-1, SIGTERM) to kill all other processes.
+     * Do not forward signals sent by a process in the command's process
+     * group, do not forward it as we don't want the child to indirectly
+     * kill itself.  For example, this can happen with some versions of
+     * reboot that call kill(-1, SIGTERM) to kill all other processes.
      */
-    if (info != NULL && info->si_code == SI_USER && info->si_pid == cmnd_pid)
-	    return;
+    if (USER_SIGNALED(info)) {
+	pid_t si_pgrp = getpgid(info->si_pid);
+	if (si_pgrp != (pid_t)-1) {
+	    if (si_pgrp == ppgrp || si_pgrp == cmnd_pid)
+		return;
+	} else if (info->si_pid == cmnd_pid) {
+		return;
+	}
+    }
 
     /*
      * The pipe is non-blocking, if we overflow the kernel's pipe
      * buffer we drop the signal.  This is not a problem in practice.
      */
-    ignore_result(write(signal_pipe[1], &signo, sizeof(signo)));
+    while (write(signal_pipe[1], &signo, sizeof(signo)) == -1) {
+	if (errno != EINTR)
+	    break;
+    }
 }
 #else
 void
@@ -880,7 +919,10 @@ handler(int s)
      * The pipe is non-blocking, if we overflow the kernel's pipe
      * buffer we drop the signal.  This is not a problem in practice.
      */
-    ignore_result(write(signal_pipe[1], &signo, sizeof(signo)));
+    while (write(signal_pipe[1], &signo, sizeof(signo)) == -1) {
+	if (errno != EINTR)
+	    break;
+    }
 }
 #endif
 
@@ -895,14 +937,31 @@ static void
 handler_user_only(int s, siginfo_t *info, void *context)
 {
     unsigned char signo = (unsigned char)s;
+    pid_t si_pgrp;
 
-    /* Only forward user-generated signals. */
-    if (info != NULL && info->si_code == SI_USER) {
-	/*
-	 * The pipe is non-blocking, if we overflow the kernel's pipe
-	 * buffer we drop the signal.  This is not a problem in practice.
-	 */
-	ignore_result(write(signal_pipe[1], &signo, sizeof(signo)));
+    /*
+     * Only forward user-generated signals not sent by a process in
+     * the command's own process group.  Signals sent by the kernel
+     * may include SIGTSTP when the user presses ^Z.  Curses programs
+     * often trap ^Z and send SIGTSTP to their own pgrp, so we don't
+     * want to send an extra SIGTSTP.
+     */
+    if (!USER_SIGNALED(info))
+	return;
+    if ((si_pgrp = getpgid(info->si_pid)) != (pid_t)-1) {
+	if (si_pgrp == ppgrp || si_pgrp == cmnd_pid)
+	    return;
+    } else if (info->si_pid == cmnd_pid) {
+	    return;
+    }
+
+    /*
+     * The pipe is non-blocking, if we overflow the kernel's pipe
+     * buffer we drop the signal.  This is not a problem in practice.
+     */
+    while (write(signal_pipe[1], &signo, sizeof(signo)) == -1) {
+	if (errno != EINTR)
+	    break;
     }
 }
 #endif /* SA_SIGINFO */
